@@ -287,7 +287,7 @@ int ext2_mount(long cons_dev, long p_offset, long quiet)
 	if (Debug) printf("ext2 block size %d\n", ext2_blocksize);
 
 	/* read in the group descriptors (immediately follows superblock) */
-	cons_read(dev, gds, ngroups * sizeof(struct ext2_group_desc),
+	cons_read(dev, gds, ngroups * group_size,
 		  partition_offset +
                   ext2_blocksize * (EXT2_MIN_BLOCK_SIZE/ext2_blocksize + 1));
 	for (i = 0; i < ngroups; i++)
