@@ -443,9 +443,37 @@ do_cdrom(int media, int kernel32, int kernel64,
 #define EXT2_HOLE	((MAXBLSIZE + 1) / EXT2_BLOCKSIZE)
 
 /* offset in bytes before start of hole,  ext2 doesn't allow holes at
- * to cover the first four blocks of the filesystem */
+ * to cover the first four blocks of the filesystem
+ *
+ * Note: modern ext2/3 has a resize_inode covering blocks 3-258 so you
+ * must either always include the -O^resize_inode when creating the
+ * filesystem or define EXT2_OFFSET to (259*EXT2_BLOCKSIZE)*/
 #define EXT2_OFFSET	(4*EXT2_BLOCKSIZE)
 
+int
+do_at_start(int init, int media, int start,
+	    int bootloaderfd, const char *commandline)
+{
+    struct firstblock f;
+
+    STRUCTREAD(media, f, 0);
+
+    if (init) {
+	fb_init(&f);
+    } else if (f.ipl_addr != start) {
+	printf("Current IPL start not consistent with disklabel, use -I to reinitialise\n");
+	error(11);
+    }
+
+    if(commandline)
+	strncpy(f.cmdline, commandline, sizeof(f.cmdline)-1);
+
+    write_bootloader(media, bootloaderfd, start,
+		     start + MAXBLSIZE, &f);
+
+    STRUCTWRITE(media, f, 0);
+}
+
 int
 do_formatted(int init, int media, const char *medianame, int partition,
 	     int f0start, int f0length, int bootloaderfd, int do_format,
@@ -502,8 +530,8 @@ do_formatted(int init, int media, const char *medianame, int partition,
 	    }
 	}
 
-	sprintf(cmd, "mke2fs %s -b %d -l %s %s", do_format == 3 ? "-j" : "",
-		EXT2_BLOCKSIZE, badblockfilename, partitionname);
+	sprintf(cmd, "mke2fs -t ext%d -O^resize_inode -b %d -l %s %s",
+		do_format, EXT2_BLOCKSIZE, badblockfilename, partitionname);
 
 	if (verbose)
 	    printf("Executing: %s\n", cmd);
@@ -687,6 +715,7 @@ static struct option Longopts[] =
     {"init-cdrom", 1, 0, 'C'},
     {"verbose", 0, 0, 'v'},
     {"help", 0, 0, '?'},
+    {"version", 0, 0, 'V'},
     {0, 0, 0, 0}
 };
 # define GETOPT(argc, argv, optstring) \
@@ -728,8 +757,7 @@ main(int argc, char *argv[])
 	assert((unsigned long)&f.ipl_addr - (unsigned long)&f == 0xf0);
     }
 
-    fputs("palo version " PALOVERSION " ", stdout);
-    puts(bld_info);
+    fputs("palo version " PALOVERSION "\n", stdout);
 
     /* do we have a -f? */
     while ((c = GETOPT(argc, argv, gargs)) != EOF)
@@ -741,6 +769,10 @@ main(int argc, char *argv[])
 	    break;
 	case '?':
 	    error(0, argv[0]);
+	    break;
+	case 'V':
+	    puts(bld_info);
+	    return 0;
 	}
     }
 
@@ -860,6 +892,8 @@ main(int argc, char *argv[])
 		format_as = 2;
 	    else if(strcmp(optarg, "3") == 0)
 		format_as = 3;
+	    else if (strcmp(optarg, "4") == 0)
+		format_as = 4;
 	    else
 		error(0, argv[0]);
 	    break;
@@ -906,6 +940,7 @@ main(int argc, char *argv[])
 	    int partitioned;
 	    int f0;
 	    struct firstblock f;
+	    int at_start = 0;	/* place ipl before first partition */
 
 	    memset(ptab, 0, sizeof ptab);
 
@@ -933,16 +968,34 @@ main(int argc, char *argv[])
 		if (ptab[f0].id == PALO_PARTITION)
 		    break;
 	    }
-	    if (f0 == MAXPARTS)
-		error(11);
+	    if (f0 == MAXPARTS) {
+		/* is the partition layout sufficient to support
+		 * iplboot before the first parition.  Allow 64
+		 * sectors for the gpt label (like we'll ever support
+		 * gpt, but just in case) allow for 8 extra sectors to
+		 * align the iplboot */
+		if (ptab[0].start > 64 + MAXBLSIZE/512 + 8)
+		    at_start = (ptab[0].start * 512 - MAXBLSIZE) & ~0xfff;
+		else
+		    error(11);
+	    }
 	    if(verbose) {
 		print_ptab_pretty(ptab, MAXPARTS);
 
-		printf("F0 partition start sector %d length %d\n",
-		    ptab[f0].start, ptab[f0].length);
+		if (at_start)
+		    printf("Placing bootloader within disk label at %d\n",
+			   at_start/512);
+		else
+		    printf("F0 partition start sector %d length %d\n",
+			   ptab[f0].start, ptab[f0].length);
 	    }
 
-	    if (format_as) {
+	    if (at_start) {
+		if (format_as)
+		    printf("No F0(palo) partition found, placing iplboot inside label\n");
+
+		do_at_start(init, media, at_start, bootloader, commandline);
+	    } else if (format_as) {
 		/* if we're going to be a formatted partition, we can't
 		 * load anything into it, so check we haven't been asked
 		 * to */
